<?php

namespace Bixo\CrmRequest\Providers;

use Illuminate\Support\ServiceProvider;

class CrmRequestServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'crm_request');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'crm_request');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerCrmRequest();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerCrmRequest()
    {
        $this->app->bind('crm_request', function($app) {
            return new CrmRequest($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('CrmRequest', 'Lavalite\CrmRequest\Facades\CrmRequest');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('bixo.crm_request', function ($app) {
            return $this->app->make('Bixo\CrmRequest\CrmRequest');
        });

                // Bind Faq to repository
        $this->app->bind(
            'Bixo\CrmRequest\Interfaces\FaqRepositoryInterface',
            \Bixo\CrmRequest\Repositories\Eloquent\FaqRepository::class
        );

        $this->app->register(\Bixo\CrmRequest\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Bixo\CrmRequest\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and crm_request's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'bixo.crm_request'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeCrmRequest::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['bixo.crm_request'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('bixo/crm_request.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/crm_request')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/crm_request')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
